@@ -2,6 +2,7 @@ from typing import Optional
 
 import strawberry
 from strawberry import UNSET
+from strawberry.scalars import JSON
 
 from phoenix.server.api.types.GenerativeProvider import GenerativeProviderKey
 
@@ -17,7 +18,7 @@ class GenerativeModelInput:
     """ The endpoint to use for the model. Only required for Azure OpenAI models. """
     api_version: Optional[str] = UNSET
     """ The API version to use for the model. """
-
-    def __post_init__(self) -> None:
-        if self.base_url and self.endpoint:
-            raise ValueError("Must not provide both a base URL and an endpoint at the same time.")
+    region: Optional[str] = UNSET
+    """ The region to use for the model. """
+    custom_headers: Optional[JSON] = UNSET
+    """ Custom headers to use for the model. """
