@@ -4,7 +4,6 @@ import importlib
 import json
 import logging
 import os
-from collections.abc import AsyncIterator, Awaitable, Callable, Iterable, Sequence
 from contextlib import AbstractAsyncContextManager, AsyncExitStack
 from dataclasses import dataclass, field
 from datetime import datetime, timedelta, timezone
@@ -14,9 +13,14 @@ from types import MethodType
 from typing import (
     TYPE_CHECKING,
     Any,
+    AsyncIterator,
+    Awaitable,
+    Callable,
+    Iterable,
     NamedTuple,
     Optional,
     Protocol,
+    Sequence,
     TypedDict,
     Union,
     cast,
@@ -55,6 +59,7 @@ from phoenix.config import (
     ENV_PHOENIX_CSRF_TRUSTED_ORIGINS,
     SERVER_DIR,
     OAuth2ClientConfig,
+    get_env_allow_external_resources,
     get_env_csrf_trusted_origins,
     get_env_database_allocated_storage_capacity_gibibytes,
     get_env_database_usage_insertion_blocking_threshold_percentage,
@@ -63,6 +68,7 @@ from phoenix.config import (
     get_env_grpc_interceptor_paths,
     get_env_host,
     get_env_host_root_path,
+    get_env_max_spans_queue_size,
     get_env_port,
     get_env_support_email,
     server_instrumentation_is_enabled,
@@ -80,15 +86,20 @@ from phoenix.server.api.context import Context, DataLoaders
 from phoenix.server.api.dataloaders import (
     AnnotationConfigsByProjectDataLoader,
     AnnotationSummaryDataLoader,
+    AverageExperimentRepeatedRunGroupLatencyDataLoader,
     AverageExperimentRunLatencyDataLoader,
     CacheForDataLoaders,
     DatasetExampleRevisionsDataLoader,
+    DatasetExamplesAndVersionsByExperimentRunDataLoader,
     DatasetExampleSpansDataLoader,
     DocumentEvaluationsDataLoader,
     DocumentEvaluationSummaryDataLoader,
     DocumentRetrievalMetricsDataLoader,
     ExperimentAnnotationSummaryDataLoader,
     ExperimentErrorRatesDataLoader,
+    ExperimentRepeatedRunGroupAnnotationSummariesDataLoader,
+    ExperimentRepeatedRunGroupsDataLoader,
+    ExperimentRepetitionCountsDataLoader,
     ExperimentRunAnnotations,
     ExperimentRunCountsDataLoader,
     ExperimentSequenceNumberDataLoader,
@@ -115,6 +126,7 @@ from phoenix.server.api.dataloaders import (
     SpanCostDetailSummaryEntriesBySpanDataLoader,
     SpanCostDetailSummaryEntriesByTraceDataLoader,
     SpanCostSummaryByExperimentDataLoader,
+    SpanCostSummaryByExperimentRepeatedRunGroupDataLoader,
     SpanCostSummaryByExperimentRunDataLoader,
     SpanCostSummaryByGenerativeModelDataLoader,
     SpanCostSummaryByProjectDataLoader,
@@ -150,6 +162,7 @@ from phoenix.server.grpc_server import GrpcServer
 from phoenix.server.jwt_store import JwtStore
 from phoenix.server.middleware.gzip import GZipMiddleware
 from phoenix.server.oauth2 import OAuth2Clients
+from phoenix.server.prometheus import SPAN_QUEUE_REJECTIONS
 from phoenix.server.retention import TraceDataSweeper
 from phoenix.server.telemetry import initialize_opentelemetry_tracer_provider
 from phoenix.server.types import (
@@ -245,6 +258,8 @@ class AppConfig(NamedTuple):
     """ Support email address for user assistance """
     has_db_threshold: bool = False
     """ Whether the database has a threshold for usage """
+    allow_external_resources: bool = True
+    """ Whether to allow external resources like Google Fonts in the web interface """
 
 
 class Static(StaticFiles):
@@ -314,6 +329,7 @@ class Static(StaticFiles):
                     "management_url": self._app_config.management_url,
                     "support_email": self._app_config.support_email,
                     "has_db_threshold": self._app_config.has_db_threshold,
+                    "allow_external_resources": self._app_config.allow_external_resources,
                 },
             )
         except Exception as e:
@@ -423,13 +439,13 @@ class Scaffolder(DaemonTask):
     def __init__(
         self,
         config: ScaffolderConfig,
-        queue_span: Callable[[Span, ProjectName], Awaitable[None]],
-        queue_evaluation: Callable[[pb.Evaluation], Awaitable[None]],
+        enqueue_span: Callable[[Span, ProjectName], Awaitable[None]],
+        enqueue_evaluation: Callable[[pb.Evaluation], Awaitable[None]],
     ) -> None:
         super().__init__()
         self._db = config.db
-        self._queue_span = queue_span
-        self._queue_evaluation = queue_evaluation
+        self._enqueue_span = enqueue_span
+        self._enqueue_evaluation = enqueue_evaluation
         self._tracing_fixtures = [
             get_trace_fixture_by_name(name) for name in set(config.tracing_fixture_names)
         ]
@@ -500,9 +516,9 @@ class Scaffolder(DaemonTask):
                 project_name = fixture.project_name or fixture.name
                 logger.info(f"Loading '{project_name}' fixtures...")
                 for span in fixture_spans:
-                    await self._queue_span(span, project_name)
+                    await self._enqueue_span(span, project_name)
                 for evaluation in fixture_evals:
-                    await self._queue_evaluation(evaluation)
+                    await self._enqueue_evaluation(evaluation)
 
             except FileNotFoundError:
                 logger.warning(f"Fixture file not found for '{fixture.name}'")
@@ -525,6 +541,32 @@ class Scaffolder(DaemonTask):
             logger.error(f"Error processing dataset fixture: {e}")
 
 
+class _CapacityIndicator(Protocol):
+    @property
+    def is_full(self) -> bool: ...
+
+
+class CapacityInterceptor(AsyncServerInterceptor):
+    def __init__(self, indicator: _CapacityIndicator):
+        self._indicator = indicator
+
+    @override
+    async def intercept(
+        self,
+        method: Callable[[Any, grpc.aio.ServicerContext], Awaitable[Any]],
+        request_or_iterator: Any,
+        context: grpc.aio.ServicerContext,
+        method_name: str,
+    ) -> Any:
+        if self._indicator.is_full:
+            SPAN_QUEUE_REJECTIONS.inc()
+            context.set_code(grpc.StatusCode.RESOURCE_EXHAUSTED)
+            context.set_details("Server is at capacity and cannot process more requests")
+            return
+
+        return await method(request_or_iterator, context)
+
+
 def _lifespan(
     *,
     db: DbSessionFactory,
@@ -551,18 +593,23 @@ def _lifespan(
         db.lock = asyncio.Lock() if db.dialect is SupportedSQLDialect.SQLITE else None
         async with AsyncExitStack() as stack:
             (
-                enqueue,
-                queue_span,
-                queue_evaluation,
+                enqueue_annotations,
+                enqueue_span,
+                enqueue_evaluation,
                 enqueue_operation,
             ) = await stack.enter_async_context(bulk_inserter)
+            interceptors = [
+                CapacityInterceptor(bulk_inserter),
+                *user_grpc_interceptors(),
+                *grpc_interceptors,
+            ]
             grpc_server = GrpcServer(
-                queue_span,
+                enqueue_span,
                 disabled=read_only,
                 tracer_provider=tracer_provider,
                 enable_prometheus=enable_prometheus,
                 token_store=token_store,
-                interceptors=user_grpc_interceptors() + list(grpc_interceptors),
+                interceptors=interceptors,
             )
             await stack.enter_async_context(grpc_server)
             await stack.enter_async_context(dml_event_handler)
@@ -574,17 +621,17 @@ def _lifespan(
             if scaffolder_config:
                 scaffolder = Scaffolder(
                     config=scaffolder_config,
-                    queue_span=queue_span,
-                    queue_evaluation=queue_evaluation,
+                    enqueue_span=enqueue_span,
+                    enqueue_evaluation=enqueue_evaluation,
                 )
                 await stack.enter_async_context(scaffolder)
             if isinstance(token_store, AbstractAsyncContextManager):
                 await stack.enter_async_context(token_store)
             yield {
                 "event_queue": dml_event_handler,
-                "enqueue": enqueue,
-                "queue_span_for_bulk_insert": queue_span,
-                "queue_evaluation_for_bulk_insert": queue_evaluation,
+                "enqueue_annotations": enqueue_annotations,
+                "enqueue_span": enqueue_span,
+                "enqueue_evaluation": enqueue_evaluation,
                 "enqueue_operation": enqueue_operation,
             }
         for callback in shutdown_callbacks:
@@ -659,9 +706,15 @@ def create_graphql_router(
             event_queue=event_queue,
             data_loaders=DataLoaders(
                 annotation_configs_by_project=AnnotationConfigsByProjectDataLoader(db),
+                average_experiment_repeated_run_group_latency=AverageExperimentRepeatedRunGroupLatencyDataLoader(
+                    db
+                ),
                 average_experiment_run_latency=AverageExperimentRunLatencyDataLoader(db),
                 dataset_example_revisions=DatasetExampleRevisionsDataLoader(db),
                 dataset_example_spans=DatasetExampleSpansDataLoader(db),
+                dataset_examples_and_versions_by_experiment_run=DatasetExamplesAndVersionsByExperimentRunDataLoader(
+                    db
+                ),
                 document_evaluation_summaries=DocumentEvaluationSummaryDataLoader(
                     db,
                     cache_map=(
@@ -680,6 +733,11 @@ def create_graphql_router(
                 ),
                 experiment_annotation_summaries=ExperimentAnnotationSummaryDataLoader(db),
                 experiment_error_rates=ExperimentErrorRatesDataLoader(db),
+                experiment_repeated_run_group_annotation_summaries=ExperimentRepeatedRunGroupAnnotationSummariesDataLoader(
+                    db
+                ),
+                experiment_repeated_run_groups=ExperimentRepeatedRunGroupsDataLoader(db),
+                experiment_repetition_counts=ExperimentRepetitionCountsDataLoader(db),
                 experiment_run_annotations=ExperimentRunAnnotations(db),
                 experiment_run_counts=ExperimentRunCountsDataLoader(db),
                 experiment_sequence_number=ExperimentSequenceNumberDataLoader(db),
@@ -736,6 +794,11 @@ def create_graphql_router(
                 span_cost_details_by_span_cost=SpanCostDetailsBySpanCostDataLoader(db),
                 span_cost_detail_fields=TableFieldsDataLoader(db, models.SpanCostDetail),
                 span_cost_fields=TableFieldsDataLoader(db, models.SpanCost),
+                span_cost_summary_by_experiment=SpanCostSummaryByExperimentDataLoader(db),
+                span_cost_summary_by_experiment_repeated_run_group=SpanCostSummaryByExperimentRepeatedRunGroupDataLoader(
+                    db
+                ),
+                span_cost_summary_by_experiment_run=SpanCostSummaryByExperimentRunDataLoader(db),
                 span_cost_summary_by_generative_model=SpanCostSummaryByGenerativeModelDataLoader(
                     db
                 ),
@@ -764,8 +827,6 @@ def create_graphql_router(
                 project_by_name=ProjectByNameDataLoader(db),
                 users=UsersDataLoader(db),
                 user_roles=UserRolesDataLoader(db),
-                span_cost_summary_by_experiment=SpanCostSummaryByExperimentDataLoader(db),
-                span_cost_summary_by_experiment_run=SpanCostSummaryByExperimentRunDataLoader(db),
             ),
             cache_for_dataloaders=cache_for_dataloaders,
             read_only=read_only,
@@ -965,11 +1026,11 @@ def create_app(
     span_cost_calculator = SpanCostCalculator(db, generative_model_store)
     bulk_inserter = bulk_inserter_factory(
         db,
-        enable_prometheus=enable_prometheus,
         span_cost_calculator=span_cost_calculator,
         event_queue=dml_event_handler,
         initial_batch_of_spans=initial_batch_of_spans,
         initial_batch_of_evaluations=initial_batch_of_evaluations,
+        max_spans_queue_size=get_env_max_spans_queue_size(),
     )
     tracer_provider = None
     graphql_schema_extensions: list[Union[type[SchemaExtension], SchemaExtension]] = []
@@ -1085,6 +1146,7 @@ def create_app(
                         get_env_database_allocated_storage_capacity_gibibytes()
                         and get_env_database_usage_insertion_blocking_threshold_percentage()
                     ),
+                    allow_external_resources=get_env_allow_external_resources(),
                 ),
             ),
             name="static",
@@ -1099,6 +1161,7 @@ def create_app(
     app.state.db = db
     app.state.email_sender = email_sender
     app.state.span_cost_calculator = span_cost_calculator
+    app.state.span_queue_is_full = lambda: bulk_inserter.is_full
     app = _add_get_secret_method(app=app, secret=secret)
     app = _add_get_token_store_method(app=app, token_store=token_store)
     if tracer_provider:
